# -*- coding: Utf-8 -*-
# Author : Stéphane DUTERTRE
# Date : 12/04/19
# Description : Management of a bit FIFO. Input data is a string in hexa format without prefix '$' or '0x'
# Version : 1.0.0.0

# les masques à appliquer
DeltaCompression = [
 [ #0
   2,
   #0,
   0b11,                #On va stocker en mémoire '0b..11'
   #0,
   0,                   # Le taille du signe
   #0,
   0,                   # La taille de la donnée
   #0
   0                    # +0
 ], 
 [ #1
   2,
   #1
   0b01,                #On va stocker en mémoire '0b.01s'   
   #0,
   1,                   # Le taille du signe
   #0,
   0,                   # La taille de la donnée
   #1
   1,                   # +2 
 ],
 [ #2
   3,
   #2
   0b101,               #On va stocker en mémoire '0b1.01sx'
   #0,
   1,                   # Le taille du signe
   #0,
   1,                   # La taille de la donnée
   #2
   2,                   # +4
 ],
 [ #3
   3,
   #3
   0b001,               #On va stocker en mémoire '0b00.1sxx'
   #4,
   1,                   # Le taille du signe
   #0,
   2,                   # La taille de la donnée
   #3
   4,                   # +8
 ],
 [ #4
   4,
   #4
   0b0001,              #On va stocker en mémoire '0b0001.sxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   3,                   # La taille de la donnée
   #4
   8,                  # +16
 ],
 [ #Valeur entière
   4,
   #4
   0b1001,              #On va stocker en mémoire '0b1001.....'
   #0,
   0,                   # Le taille du signe
   #0,
   0,                   # La taille de la donnée
   #4
   0,                   # +0
 ],
 [ #5
   5,
   #5
   0b00001,             #On va stocker en mémoire '0b00.001s.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   4,                   # La taille de la donnée
   #5
   16,                  # +32
 ],
 [ #6
   5,
   #6
   0b10001,             #On va stocker en mémoire '0b100.01sx.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   5,                   # La taille de la donnée
   #4
   32,                  # +64
 ],
 [ #7
   6,
   #7
   0b000001,            #On va stocker en mémoire '0b0.0000.1sxx.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   6,                   # La taille de la donnée
   #4
   64,                 # +128
 ],
 [ #8
   6,
   #8
   0b100001,            # On va stocker en mémoire '0b10.0001.sxxx.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   7,                   # La taille de la donnée
   #8
   128,                 # +256
 ],
 [ #9
   7,
   #9
   0b0000001,           #On va stocker en mémoire '0b0000.001s.xxxx.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   8,                   # La taille de la donnée
   #9
   256,                 # +512
 ],
 [ #10
   8,
   #10
   0b10000010,          #On va stocker en mémoire '0b10.0000.10sx.xxxx.xxxx'
   #10,
   1,                   # Le taille du signe
   #0,
   9,                   # La taille de la donnée
   #10
   512,                # +1024
 ],
 [ #11
   8,
   #11
   0b10000011,          #On va stocker en mémoire '0b100.0001.1sxx.xxxx.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   10,                  # La taille de la donnée
   #11
   1024,                # +2048
 ],
 [ #12
   9,
   #12
   0b000000010,         #On va stocker en mémoire '0b0.0000.0010.sxxx.xxxx.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   11,                   # La taille de la donnée
   #12
   2048,                # +4096
 ],
 [ #13
   9,
   #13
   0b000000011,         #On va stocker en mémoire '0b00.0000.011s.xxxx.xxxx.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   12,                  # La taille de la donnée
   #13
   4096,                # +8192
 ],
 [ #14
   9,
   #14
   0b100000010,         #On va stocker en mémoire '0b100.0000.10sx.xxxx.xxxx.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   13,                  # La taille de la donnée
   #14
   8192,               # +16384
 ],
 [ #15
   9,
   #15
   0b100000011,         #On va stocker en mémoire '0b1000.0001.1sxx.xxxx.xxxx.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   14,                   # La taille de la donnée
   #15
   16384,               # +32768
 ],
 [ #16
   9,
   #16
   0b100000000,         #On va stocker en mémoire '0b1.0000.0000.sxxx.xxxx.xxxx.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   15,                   # La taille de la donnée
   #16
   32768,               # +65536
 ],
 [ #17
   9,  
   #17
   0b000000000,         #On va stocker en mémoire '0b00.0000.000s.xxxx.xxxx.xxxx.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   16,                  # La taille de la donnée
   #17
   65536,              # +131072
 ],
 [ #18
   10,
   #18
   0b1000000011,        #On va stocker en mémoire '0b1000.0000.11sx.xxxx.xxxx.xxxx.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   17,                   # La taille de la donnée
   #18
   131072,              #262144
 ],
 [ #19
   10,
   #19
   0b0000000010,        #On va stocker en mémoire '0b0.0000.0001.0sxx.xxxx.xxxx.xxxx.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   18,                  # La taille de la donnée
   #19
   262144,  #524288
 ],
 [ #20
   11,
   #20
   0b10000000100,       #On va stocker en mémoire '0b100.0000.0100.sxxx.xxxx.xxxx.xxxx.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   19,                  # La taille de la donnée
   #20
   524288,  #1048576
 ],
 [ #21
   11,
   #21
   0b10000000101,       #On va stocker en mémoire '0b1000.0000.101s.xxxx.xxxx.xxxx.xxxx.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   20,                  # La taille de la donnée
   #21
   1048576,  #2097152
 ],
 [ #22
   11,
   #22
   0b00000000110,       #On va stocker en mémoire '0b0.0000.0001.10sx.xxxx.xxxx.xxxx.xxxx.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   21,                  # La taille de la donnée
   #22
   2097152,  #4194304
 ],
 [ #23
   11,
   #23
   0b00000000111,       #On va stocker en mémoire '0b00.0000.0011.1sxx.xxxx.xxxx.xxxx.xxxx.xxxx'
   #4,
   1,                   # Le taille du signe
   #0,
   22,                  # La taille de la donnée
   #23
   4194304,              #8388608
 ]  
]
# le nombre de bit de utilisable de l'opcode
DeltaCompressionOpcodeBitLength = 11

class SigfoxFifoObject :
    def __init__( self, Encoded = '' ) :
        if( ( len( Encoded ) % 2 ) != 0 ) :
            raise 'Length of the string must a multiple of 2 digits'
        self.Encoded = Encoded
        self.Counter = 0
        self.BitCount = 0
    def __rep__( self ) :
        return "Encoded Payload = " + self.Encoded
    def Initialization( self, Encoded = '' ) :
        if( ( len( Encoded ) % 2 ) != 0 ) :
            raise 'Length of the string must a multiple of 2 digits'
        self.Encoded = Encoded
        self.Counter = 0
        self.BitCount = 0
    def ReWind( self ) :
        self.Counter = 0
        self.BitCount = 0
    def String( self ) :
        return self.Encoded
    def IsEnded( self ) :
        if( self.Counter >= len( self.Encoded ) ) :
            return True
        else :
            return False
    def __add__( self, Encoded ) :
        self.Encoded.append( Encoded )
    def Pop( self ) :
        if( self.Counter < ( len( self.Encoded ) ) ) :
            Index = int( self.Counter )
            Byte = int( '0x' + self.Encoded[ Index ], 16 ) * 16 + int( '0x' + self.Encoded[ Index + 1 ], 16 )
            Data = Byte & ( 1 << ( self.BitCount % 8 ) )
            self.BitCount += 1
            if( ( self.BitCount % 8 ) == 0 ) :
                self.BitCount = 0
                self.Counter  += 2
            return bool( Data > 0 )
        return False
    def Size( self ) :
        return len( self.Encoded ) / 2
