#-*- coding: Utf-8 -*-
#-----------------------------------------------------------------------------#
#                           DESCRIPTION - Initiale                            #
#-----------------------------------------------------------------------------#
# Author : Stéphane DUTERTRE
# Date : 12/04/19
# Description : Decodes Sigfox payload
# Version : 1.0.0
#-----------------------------------------------------------------------------#
#                                  Correctif                                  #
#-----------------------------------------------------------------------------#
# revision : 1.0.1
# Author : Stéphane DUTERTRE
# Date : 30/04/19 
# Description :
# > Modification de la fonction membre isFullyLoaded. 
# > Si l'object est vide alors, elle renvoie maintenant faux
#-----------------------------------------------------------------------------#
#                                  Correctif                                  #
#-----------------------------------------------------------------------------#
# revision : 1.0.2
# Author : Stéphane DUTERTRE
# Date : 05/06/19 
# Description :
# > Affiche les données antéchronologique dans le même ordre que chronologiques
# dans la liste
#-----------------------------------------------------------------------------#
#                                  Correctif                                  #
#-----------------------------------------------------------------------------#
# revision : 1.0.3
# Author : Patrice LE GARFF
# Date : 15/09/21 
# Description :
# > Problème de test du mode chrono/antéchrono inversé
# dans la liste
#-----------------------------------------------------------------------------#
#                                  Correctif                                  #
#-----------------------------------------------------------------------------#
# revision : 1.0.4
# Author : Patrice LE GARFF
# Date : 22/09/21 
# Description :
# > Problème de décodage des trames TH compressées
#-----------------------------------------------------------------------------#
SigfoxCODECVersion = '1.0.4'

from SigfoxRawDecoder_1_00 import SigfoxRawDecoderObject
from SigfoxZipDecoder_1_00 import SigfoxZipDecoderObject

# les types de produits
SigfoxProductTypeVentilation = 200
SigfoxProductTypeTOutdoor    = 201
SigfoxProductTypeTH          = 202
SigfoxProductTypeS0          = 203
SigfoxProductTypeAlarmReport = 204
SigfoxProductTypeTIndoor     = 205
#SigfoxProductTypeFree       = 206
SigfoxProductTypeInputstate  = 207
SigfoxProductTypeCMV         = 208
SigfoxProductTypeSkydome     = 209
SigfoxProductType20mALoop_0_10V = 210 # Data non gérées pour l'instant !

# les types de messages
SigfoxMessageTypeStandardData    = 0
SigfoxMessageTypeExtendedData    = 1
SigfoxMessageTypAlarm            = 2
SigfoxMessageTypeInformationData = 3

# les types de compression
SigfoxMessageTypeCompressionNone      = 0
SigfoxMessageTypeCompressionStandard  = 1

# les types d'alarmes
SigfoxAlarmTypeActivation      = 0
SigfoxAlarmTypeLowVoltage      = 1
SigfoxAlarmTypeLowTemperature  = 2
SigfoxAlarmTypeHighTemperature = 3
SigfoxAlarmTypeLowPressure     = 4
SigfoxAlarmTypeHighPressure    = 5
SigfoxAlarmTypeLowHygrometry   = 6
SigfoxAlarmTypeHighHygrometry  = 7
SigfoxAlarmTypeInputState      = 8
SigfoxAlarmTypeMalfunction     = 9
SigfoxalarmTypeTearingState    = 10
SigfoxalarmTypeToggleState     = 11
SigfoxalarmTypeMeasureError    = 12
SigfoxalarmTypeExternalInput   = 13
SigfoxalarmTypeOpeningState    = 14
SigfoxAlarmTypeLowAnalogValue  = 16
SigfoxAlarmTypeHighAnalogValue = 17
SigfoxAlarmTypeMax             = 18

# this class is able to extract compressed data
class SigfoxPayloadObject :
        __SigfoxRawDecoder = SigfoxRawDecoderObject()
        __SigfoxZipDecoder = SigfoxZipDecoderObject()
        __ProductNameList  = [ [ SigfoxProductTypeVentilation, 'Ventilation sensor'            ],
                               [ SigfoxProductTypeTOutdoor   , 'Outdoor temperature sensor'    ], 
                               [ SigfoxProductTypeTH         , 'Temperature-hygrometry sensor' ], 
                               [ SigfoxProductTypeS0         , 'S0 sensor'                     ], 
                               [ SigfoxProductTypeAlarmReport, 'Alarm report sensor'           ], 							   
                               [ SigfoxProductTypeTIndoor    , 'Indoor temperature sensor'     ], 
                               #[ SigfoxProductTypeFree       , 'Free sensor'                   ], 
                               [ SigfoxProductTypeCMV        , 'CMV-Gaz sensor'                ],
                               [ SigfoxProductTypeInputstate , 'Input state sensor'            ],
                               [ SigfoxProductTypeSkydome    , 'Skydome sensor'                ],							   
                               [ SigfoxProductType20mALoop_0_10V , '20mA Loop 0-10V sensor'    ] ]
        __MessageNameList  = [ [ SigfoxMessageTypeStandardData    , 'Standard'    ],
                               [ SigfoxMessageTypeExtendedData    , 'Extended'    ], 
                               [ SigfoxMessageTypAlarm            , 'Alarm'       ], 
                               [ SigfoxMessageTypeInformationData , 'Information' ] ]
        __AlarmNameList    = [ [ SigfoxAlarmTypeActivation      , 'Activation state'     ],
                               [ SigfoxAlarmTypeLowVoltage      , 'Low battery voltage'  ], 
                               [ SigfoxAlarmTypeLowTemperature  , 'Low temperature'      ], 
                               [ SigfoxAlarmTypeHighTemperature , 'High temperature'     ], 
                               [ SigfoxAlarmTypeLowPressure     , 'Low pressure'         ], 
                               [ SigfoxAlarmTypeHighPressure    , 'High pressure'        ], 
                               [ SigfoxAlarmTypeLowHygrometry   , 'Low hygrometry'       ], 
                               [ SigfoxAlarmTypeHighHygrometry  , 'High hygrometry'      ], 
                               [ SigfoxAlarmTypeInputState      , 'Input state'          ], 
                               [ SigfoxAlarmTypeMalfunction     , 'Malfunction state'    ], 
                               [ SigfoxalarmTypeTearingState    , 'Tearing state'        ],
                               [ SigfoxalarmTypeToggleState     , 'Toggle state'         ],
                               [ SigfoxalarmTypeMeasureError    , 'Measure error'        ],
                               [ SigfoxalarmTypeExternalInput   , 'External input state' ],
                               [ SigfoxalarmTypeOpeningState    , 'Opening state'        ],
                               [ SigfoxAlarmTypeLowAnalogValue  , 'Low analog value'     ], 
                               [ SigfoxAlarmTypeHighAnalogValue , 'High analog value'    ] ]
        #---------------------------------------------------------------------------------------------------------------
        # Just to print correctly an instance of "SigfoxPayloadObject"
        #---------------------------------------------------------------------------------------------------------------
        def __repr__( self ) :
                return 'nkeWatteco Sigfox payload decoder - Version n°' + SigfoxCODECVersion
        #---------------------------------------------------------------------------------------------------------------
        # Constructor of an instance of "SigfoxPayloadObject"
        #---------------------------------------------------------------------------------------------------------------
        def __init__( self, PayLoad = "" ) :
                self.__TimeUsed        = False
                self.__ProductType     = None
                self.__MessageType     = None
                self.__CompressionType = None
                self.__FrameIndex      = None
                self.__MeasurePeriod   = None
                self.__InformationType = None
                self.__MeasureReverse  = None
                self.__AlarmType       = None
                self.__Data            = []
                self.__Array           = []
                self.__InputString     = ''
                self.Initialization( PayLoad )
        #---------------------------------------------------------------------------------------------------------------
        # This function initiliazes the payload with the 1st sigfox received frame and returns true on success
        #---------------------------------------------------------------------------------------------------------------
        def Initialization( self, PayLoad = "" ) :
            # On vérifie la cohérence
            if(    ( PayLoad        != None )
               and ( len( PayLoad ) >= 8    ) ) :
                # on gère le cas particulier du VMC Gaz 1ère version
                if len( PayLoad ) == 14 :
                    # on regarde si le type est cohérent
                    ProductType = ( int( '0x' + PayLoad[ 4 ], 16 ) * 16 + int( '0x' + PayLoad[ 5 ], 16 ) )
                    # on regarde si le type est connu
                    #--------------------------------------------------------------------------------------------------
                    # Ventilation Sensor
                    #--------------------------------------------------------------------------------------------------
                    if ProductType == SigfoxProductTypeVentilation :
                            # on va aussi récupérer le Msg pour vérifier sa validité
                            MessageType = ( int( '0x' + PayLoad[ 6 ], 16 ) * 16 + int( '0x' + PayLoad[ 7 ], 16 ) )
                            # on teste sa cohérence
                            if MessageType == SigfoxMessageTypeStandardData :
                                    # on mémorise tous les champs
                                    self.__ProductType     = ProductType
                                    self.__MessageType     = MessageType
                                    # ici il n'y pas de notion de compression
                                    self.__CompressionType = SigfoxMessageTypeCompressionNone
                                    self.__FrameIndex      = None
                                    self.__MeasurePeriod   = None
                                    self.__MeasureReverse  = None 
                                    # c'est un message sans flux de données
                                    self.__TimeUsed        = False
                                    # On note le payload
                                    self.__InputString = PayLoad[ : ]
                                    # On va vérifier que la tension de pile est cohérente
                                    self.__Data.append( [] )
                                    self.__Data[ 0 ].append( [ 'Pa' ] )
                                    self.__Data[ 0 ].append( 16 )
                                    self.__Data[ 0 ].append( 0 )
                                    self.__Data[ 0 ].append( PayLoad[ 8 : 12 ] )
                                    self.__Data[ 0 ].append( 1 )
                                    self.__Data[ 0 ].append( [ 'Mean Pressure' ] )
                                    self.__Data.append( [] )
                                    self.__Data[ 1 ].append( ['V' ] )
                                    self.__Data[ 1 ].append( 8 )
                                    self.__Data[ 1 ].append( 0 )
                                    self.__Data[ 1 ].append( PayLoad[ 12 : 14 ] )
                                    self.__Data[ 1 ].append( 0.1 )
                                    self.__Data[ 1 ].append( [ 'Battery Voltage' ] )
                                    self.__Data.append( [] )
                                    self.__Data[ 2 ].append( [ 'Count' ] )
                                    self.__Data[ 2 ].append( 16 )
                                    self.__Data[ 2 ].append( 0 )
                                    self.__Data[ 2 ].append( PayLoad[ 0 : 4 ] )
                                    self.__Data[ 2 ].append( 1 )
                                    self.__Data[ 2 ].append( [ 'Frame Number' ] )
                                    # on sort ici
                                    return True
                # on récupère le type produit
                ProductType     = ( int( '0x' + PayLoad[ 0 ], 16 ) * 16 + int( '0x' + PayLoad[ 1 ], 16 ) )
                Byte            = ( int( '0x' + PayLoad[ 2 ], 16 ) * 16 + int( '0x' + PayLoad[ 3 ], 16 ) )
                MessageType     = ( Byte & 0x03 )
                CompressionType = ( Byte & 0x0C ) >> 2
                FrameIndex      = ( Byte & 0xF0 ) >> 4
                # On vérifie que l'on connait le produit et que la trame est cohérente 1ère trame
                if FrameIndex == 0 :
                    # On note le payload
                    self.__InputString = PayLoad[ : ]
                    # on vérifie que l'on connait le produit
                    if(   ( ProductType == SigfoxProductTypeTOutdoor    )
                       or ( ProductType == SigfoxProductTypeTH          )
                       or ( ProductType == SigfoxProductTypeS0          )
                       or ( ProductType == SigfoxProductTypeAlarmReport )
                       or ( ProductType == SigfoxProductTypeTIndoor     )
                       #or ( ProductType == SigfoxProductTypeFree       )	
                       or ( ProductType == SigfoxProductTypeInputstate  ) 					   
                       or ( ProductType == SigfoxProductTypeCMV         ) 
                       or ( ProductType == SigfoxProductTypeSkydome     )
                       or ( ProductType == SigfoxProductType20mALoop_0_10V ) ) :
                        # on vérifie aussi le type de compression
                        if(   ( CompressionType == SigfoxMessageTypeCompressionNone     )
                           or ( CompressionType == SigfoxMessageTypeCompressionStandard ) ) :
                            # on mémorise tous les champs
                            self.__ProductType     = ProductType
                            self.__MessageType     = MessageType
                            self.__CompressionType = CompressionType
                            self.__FrameIndex      = FrameIndex						   
                            #--------------------------------------------------------------------------------------------------
                            # Standard Message
                            #--------------------------------------------------------------------------------------------------
                            # on vérifie qu'il s'agit bien d'un message standard
                            if( MessageType == SigfoxMessageTypeStandardData ) :
                                # on note aussi le message
                                self.Function        = ( int( '0x' + PayLoad[ 4 ], 16 ) * 16 + int( '0x' + PayLoad[ 5 ], 16 ) )
                                # on note la période de mesure
                                self.__MeasurePeriod   = ( int( '0x' + PayLoad[ 6 ], 16 ) * 16 + int( '0x' + PayLoad[ 7 ], 16 ) )
                                # on regarde si l'on inverse ou pas
                                if( ProductType == SigfoxProductTypeCMV ) :
                                    self.__MeasureReverse = False
                                else :
                                    self.__MeasureReverse  = bool( ( self.__MeasurePeriod & 0x80 ) == 0 )
                                # on regarde si l'on est en minute ou en heure
                                if( ( self.__MeasurePeriod & 0x40 ) > 0 ) :
                                        self.__MeasurePeriod = ( self.__MeasurePeriod & 0x3F ) * 60
                                else :
                                        self.__MeasurePeriod = ( self.__MeasurePeriod & 0x3F ) * 3600
                                # on efface les précédentes listes
                                self.__Data.clear()
                                self.__Array.clear()
                                #--------------------------------------------------------------------------------------------------
                                # Temperature Sensor
                                #--------------------------------------------------------------------------------------------------
                                # on regarde suivant les cas
                                if(   ( self.__ProductType == SigfoxProductTypeTOutdoor ) 
                                   or ( self.__ProductType == SigfoxProductTypeTIndoor  ) ) :
                                    # on va devoir gérer un flux de données
                                    self.__TimeUsed = True
                                    # on note la taille du buffer de données
                                    self.__Data.append( [] )
                                    # unités des données
                                    self.__Data[ 0 ].append( [ 's', '°C' ] )
                                    # taille de la donnée
                                    self.__Data[ 0 ].append( 16 )
                                    # nombre de mesures									
                                    self.__Data[ 0 ].append( int( '0x' + PayLoad[ 8 ], 16 ) * 16 + int( '0x' + PayLoad[ 9 ], 16 ) )
                                    # données brutes
                                    self.__Data[ 0 ].append( PayLoad[ 10: ] )
                                    # mise à jour du nombre de mesures
                                    self.__Data[ 0 ][ 2 ] -= int( len( self.__Data[ 0 ][ 3 ] ) / 2 )
                                    # facteur de conversion
                                    self.__Data[ 0 ].append( 0.1 )
                                    # labels des données
                                    self.__Data[ 0 ].append( [ 'TimeStamp', 'Temperature' ] )
                                #--------------------------------------------------------------------------------------------------
                                # Temperature-Hygrometry Sensor
                                #--------------------------------------------------------------------------------------------------
                                elif( self.__ProductType == SigfoxProductTypeTH ) :
                                    # on va devoir gérer un flux de données
                                    self.__TimeUsed = True
                                    #print( PayLoad[ : ] )
                                    # on note la taille du buffer de données
                                    ## Température ##
                                    self.__Data.append( [] )
                                    # unités des données
                                    self.__Data[ 0 ].append( [ 's', '°C' ] )
                                    # taille de la donnée
                                    self.__Data[ 0 ].append( 16 )
                                    # nombre de mesures
                                    self.__Data[ 0 ].append( int( '0x' + PayLoad[ 8 ], 16 ) * 16 + int( '0x' + PayLoad[ 9 ], 16 ) )
                                    # données brutes
                                    i = (self.__Data[ 0 ][ 2 ]) * 2
                                    #print( i )	
                                    self.__Data[ 0 ].append( PayLoad[ 12:12 + i ] )
                                    # mise à jour du nombre de mesures
                                    self.__Data[ 0 ][ 2 ] -= int( len( self.__Data[ 0 ][ 3 ] ) / 2 )
                                    #self.__Data[ 0 ][ 2 ] -= ( i / 2 )
                                    #print( "0 manque " + str( self.__Data[ 0 ][ 2 ] ) )
                                    # facteur de conversion
                                    self.__Data[ 0 ].append( 0.1 )
                                    # labels des données
                                    self.__Data[ 0 ].append( [ 'TimeStamp', 'Temperature' ] )
                                    ## Hygrométrie ##
                                    self.__Data.append( [] )
                                    # unités des données
                                    self.__Data[ 1 ].append( [ 's', '%RH' ] )
                                    # taille de la donnée
                                    self.__Data[ 1 ].append( 16 )
                                    # nombre de mesures
                                    self.__Data[ 1 ].append( int( '0x' + PayLoad[ 10 ], 16 ) * 16 + int( '0x' + PayLoad[ 11 ], 16 ) )
                                    # données brutes
                                    #self.__Data[ 1 ].append( '' )
                                    self.__Data[ 1 ].append( PayLoad[ 12 + i: ] )
                                    #print( self.__Data[ 1 ][ 3 ] )
                                    # mise à jour du nombre de mesures
                                    self.__Data[ 1 ][ 2 ] -= int( len( self.__Data[ 1 ][ 3 ] ) / 2 )
                                    #print( "1 manque " + str(self.__Data[ 1 ][ 2 ] ) )
                                    # facteur de conversion
                                    self.__Data[ 1 ].append( 0.1 )
                                    # labels des données
                                    self.__Data[ 1 ].append( [ 'TimeStamp', 'Hygrometry' ] )
                                #--------------------------------------------------------------------------------------------------
                                # S0 Sensor
                                #--------------------------------------------------------------------------------------------------
                                elif( self.__ProductType == SigfoxProductTypeS0 ) :
                                    # on va devoir gérer un flux de données
                                    self.__TimeUsed = True
                                    # on note la taille du buffer de données
                                    self.__Data.append( [] )
                                    # unités des données
                                    self.__Data[ 0 ].append( [ 's', 'Count' ] )
                                    # taille de la donnée
                                    self.__Data[ 0 ].append( 32 )
                                    # nombre de mesures
                                    self.__Data[ 0 ].append( int( '0x' + PayLoad[ 8 ], 16 ) * 16 + int( '0x' + PayLoad[ 9 ], 16 ) )
                                    # données brutes
                                    #self.__Data[ 0 ].append( PayLoad[ 8: ] )
                                    self.__Data[ 0 ].append( PayLoad[ 10: ] )
                                    # mise à jour du nombre de mesures
                                    self.__Data[ 0 ][ 2 ] -= int( len( self.__Data[ 0 ][ 3 ] ) / 2 )
                                    # facteur de conversion
                                    self.__Data[ 0 ].append( 1 )
                                    # labels des données
                                    self.__Data[ 0 ].append( [ 'TimeStamp', 'S0' ] )
                                #--------------------------------------------------------------------------------------------------
                                # CMV-Gaz Sensor
                                #--------------------------------------------------------------------------------------------------
                                elif( self.__ProductType == SigfoxProductTypeCMV ) :
                                    # on va devoir gérer un flux de données
                                    self.__TimeUsed = False
                                    # on note la taille du buffer de données
                                    ## => la pression Moy ##
                                    self.__Data.append( [] )
                                    # unités des données
                                    if( int( '0x' + PayLoad[ 9 ], 16 ) == 6 ) :
                                        self.__Data[ 0 ].append( [ 'Pa' ] )
                                    else :
                                        self.__Data[ 0 ].append( [ 'Pa - deg C' ] )
                                    # taille de la donnée
                                    self.__Data[ 0 ].append( 16 )
                                    # nombre de mesures
                                    #if( int( '0x' + PayLoad[ 9 ], 16 ) == 6 ) :
                                    #    self.__Data[ 0 ].append( 0 )
                                    #else :
                                    self.__Data[ 0 ].append( int( '0x' + PayLoad[ 8 ], 16 ) * 16 + int( '0x' + PayLoad[ 9 ], 16 ) )
                                    # print( ( int( '0x' + PayLoad[ 8 ], 16 ) * 16 + int( '0x' + PayLoad[ 9 ], 16 ) ) / 2 )
                                    # données brutes
                                    if( int( '0x' + PayLoad[ 9 ], 16 ) == 6 ) :
                                        self.__Data[ 0 ].append( PayLoad[ 10:22 ] )
                                    else :
                                        self.__Data[ 0 ].append( PayLoad[ 10:24 ] )
                                    #self.__Data[ 0 ].append( PayLoad[ 10 : 14 ] )
                                    #self.__Data[ 0 ].append( PayLoad[ 10 : 12 ] )
                                    # mise à jour du nombre de mesures
                                    self.__Data[ 0 ][ 2 ] -= int( len( self.__Data[ 0 ][ 3 ] ) / 2 )
                                    # facteur de conversion
                                    self.__Data[ 0 ].append( 1 )
                                    # labels des données
                                    if( int( '0x' + PayLoad[ 9 ], 16 ) == 6 ) :
                                        self.__Data[ 0 ].append( [ 'Mean Max Min Pressure' ] )
                                    else :
                                        self.__Data[ 0 ].append( [ 'Mean Max Min Pressure - Mean Max Min Temperature' ] )
                                    ## => la pression Max ##
                                    #self.__Data.append( [] )
                                    # unités des données
                                    #self.__Data[ 1 ].append( [ 'Pa' ] )
                                    # taille de la donnée
                                    #self.__Data[ 1 ].append( 16 )
                                    # nombre de mesures
                                    #self.__Data[ 1 ].append( 0 )
                                    # données brutes
                                    #self.__Data[ 1 ].append( PayLoad[ 12 : 14 ] )
                                    #self.__Data[ 1 ].append( PayLoad[ 14 : 18 ] )
                                    # facteur de conversion
                                    #self.__Data[ 1 ].append( 1 )
                                    # labels des données
                                    #self.__Data[ 1 ].append( [ 'Max Pressure' ] )
                                    ## => la pression Min ##
                                    #self.__Data.append( [] )
                                    # unités des données
                                    #self.__Data[ 2 ].append( [ 'Pa' ] )
                                    # taille de la donnée
                                    #self.__Data[ 2 ].append( 16 )
                                    # nombre de mesures
                                    #self.__Data[ 2 ].append( 0 )
                                    # données brutes
                                    #self.__Data[ 2 ].append( PayLoad[ 14 : 16 ] )
                                    #self.__Data[ 2 ].append( PayLoad[ 18 : 22 ] )
                                    # facteur de conversion
                                    #self.__Data[ 2 ].append( 1 )
                                    # labels des données
                                    #self.__Data[ 2 ].append( [ 'Min Pressure' ] )
                                    # attention les données sont sur 2 trames !!!
                                    #if( int( '0x' + PayLoad[ 9 ], 16 ) == 12 ) :
                                        ## => la température Moy ##
                                    #    self.__Data.append( [] )
                                        # unités des données
                                    #    self.__Data[ 3 ].append( [ '°C' ] )
                                        # taille de la donnée
                                    #    self.__Data[ 3 ].append( 16 )
                                        # nombre de mesures	
                                        # self.__Data[ 3 ].append( 0 )
                                    #    self.__Data[ 3 ].append( 6 )
                                        # données brutes
                                    #    self.__Data[ 3 ].append( PayLoad[ 22 : 26 ] )
                                        # facteur de conversion
                                    #    self.__Data[ 3 ].append( 0.1 )
                                        # labels des données
                                    #    self.__Data[ 3 ].append( [ 'Mean Temperature' ] )
                                        ## => la température Max ##
                                    #    self.__Data.append( [] )
                                        # unités des données
                                    #    self.__Data[ 4 ].append( [ '°C' ] )
                                        # taille de la donnée
                                    #    self.__Data[ 4 ].append( 16 )
                                        # nombre de mesures
                                    #    self.__Data[ 4 ].append( 0 )
                                        # données brutes
                                    #    self.__Data[ 4 ].append( PayLoad[ 26 : 30 ] )
                                        # facteur de conversion
                                    #    self.__Data[ 4 ].append( 0.1 )
                                        # labels des données
                                    #    self.__Data[ 4 ].append( [ 'Max Temperature' ] )
                                        ## => la température Min ##
                                    #    self.__Data.append( [] )
                                        # unités des données
                                    #    self.__Data[ 5 ].append( [ '°C' ] )
                                        # taille de la donnée
                                    #    self.__Data[ 5 ].append( 16 )
                                        # nombre de mesures
                                    #    self.__Data[ 5 ].append( 0 )
                                        # données brutes
                                    #    self.__Data[ 5 ].append( PayLoad[ 30 : 34 ] )
                                        # facteur de conversion
                                    #    self.__Data[ 5 ].append( 0.1 )
                                        # labels des données
                                    #    self.__Data[ 5 ].append( [ 'Min Temperature' ] )
                                        # print( PayLoad[ 22 ] )
                                        # print( PayLoad[ 26 ] )
                                        # print( PayLoad[ 30 ] )
                                #--------------------------------------------------------------------------------------------------
                                # 20mA loop - 0/10V Sensor
                                #--------------------------------------------------------------------------------------------------
                                #elif(   ( self.__ProductType == SigfoxProductType20mALoop_0_10V ) :
                                #    # on va devoir gérer un flux de données
                                #    self.__TimeUsed = True
                                #    # on note la taille du buffer de données
                                #    self.__Data.append( [] )
                                     # unités des données
                                #    self.__Data[ 0 ].append( [ 's', 'ADC pts' ] )
                                     # taille de la donnée
                                #    self.__Data[ 0 ].append( 16 )
                                     # nombre de mesures
                                #    self.__Data[ 0 ].append( int( '0x' + PayLoad[ 8 ], 16 ) * 16 + int( '0x' + PayLoad[ 9 ], 16 ) )
                                     # données brutes
                                #    self.__Data[ 0 ].append( PayLoad[ 10: ] )
                                     # facteur de conversion
                                #    self.__Data[ 0 ].append( 1 )
                                     # labels des données
                                #    self.__Data[ 0 ].append( [ 'TimeStamp', 'ADC pts' ] )
                                #--------------------------------------------------------------------------------------------------									
                                else :
                                    raise "Unmanaged product type (" + str( self.__ProductType ) + ")"
                            #--------------------------------------------------------------------------------------------------
                            # Information Message
                            #--------------------------------------------------------------------------------------------------
                            # on vérifie qu'il s'agit d'une trame d'informations
                            elif( MessageType == SigfoxMessageTypeInformationData ) :
                                # Ici il n'y a pas de flux de données
                                self.__TimeUsed = False
                                # on décode les données
                                self.__InformationType = ( int( '0x' + PayLoad[ 4 ], 16 ) * 16 + int( '0x' + PayLoad[ 5 ], 16 ) )
                                ## Battery voltage ##
                                self.__Data.append( [] )
                                # unités des données
                                self.__Data[ 0 ].append( [ 'V' ] )
                                # taille de la donnée
                                self.__Data[ 0 ].append( 8 )
                                # nombre de mesures
                                self.__Data[ 0 ].append( 0 )
                                # données brutes
                                self.__Data[ 0 ].append( PayLoad[ 6 : 8 ] )
                                # facteur de conversion
                                self.__Data[ 0 ].append( 0.1 )
                                # labels des données
                                self.__Data[ 0 ].append( [ 'Battery Voltage' ] )
                                ## Radio transmission count ##
                                self.__Data.append( [] )
                                # unités des données
                                self.__Data[ 1 ].append( [ 'Count' ] )
                                # taille de la donnée
                                self.__Data[ 1 ].append( 16 )
                                # nombre de mesures
                                self.__Data[ 1 ].append( 0 )
                                # données brutes
                                self.__Data[ 1 ].append( PayLoad[ 8 : 12 ] )
                                # facteur de conversion
                                self.__Data[ 1 ].append( 1 )
                                # labels des données
                                self.__Data[ 1 ].append( [ 'Frame Number' ] )
                                ## Firmware version ##
                                self.__Data.append( [] )
                                # unités des données
                                self.__Data[ 2 ].append( [ 'Version' ] )
                                # taille de la donnée
                                self.__Data[ 2 ].append( 0 )
                                # nombre de mesures
                                self.__Data[ 2 ].append( ( int( '0x' + PayLoad[ 12 ], 16 ) * 16 + int( '0x' + PayLoad[ 13 ], 16 ) ) * 100 + int( '0x' + PayLoad[ 14 ], 16 ) * 16 + int( '0x' + PayLoad[ 15 ], 16 ) )
                                # données brutes
                                self.__Data[ 2 ].append( PayLoad[ 12 : 16 ] )
                                # facteur de conversion
                                self.__Data[ 2 ].append( 0.01 )
                                # labels des données
                                self.__Data[ 2 ].append( [ 'Firmware' ] )
                                ## Configuration version ##
                                self.__Data.append( [] )
                                # unités des données
                                self.__Data[ 3 ].append( [ 'Version' ] )
                                # taille de la donnée
                                self.__Data[ 3 ].append( 0 )
                                # nombre de mesures
                                self.__Data[ 3 ].append( ( int( '0x' + PayLoad[ 16 ], 16 ) * 16 + int( '0x' + PayLoad[ 17 ], 16 ) ) * 100 + int( '0x' + PayLoad[ 18 ], 16 ) * 16 + int( '0x' + PayLoad[ 19 ], 16 ) )
                                # données brutes
                                self.__Data[ 3 ].append( PayLoad[ 16 : 20 ] )
                                # facteur de conversion
                                self.__Data[ 3 ].append( 0.01 )
                                # labels des données
                                self.__Data[ 3 ].append( [ 'Configuration' ] )
                            #--------------------------------------------------------------------------------------------------
                            # Alarm Message
                            #--------------------------------------------------------------------------------------------------
                            # on vérifie qu'il s'agit d'une trame d'alarme
                            elif( MessageType == SigfoxMessageTypAlarm ) :
                                # on récupère l'alarme
                                self.__AlarmType = ( int( '0x' + PayLoad[ 4 ], 16 ) * 16 + int( '0x' + PayLoad[ 5 ], 16 ) )
                                if self.__AlarmType < SigfoxAlarmTypeMax :
                                    # on cherche le type d'alarme générer
                                    self.__Data.append( [] )
                                    self.__Data[ 0 ].append( [ 'Status' ] )
                                    self.__Data[ 0 ].append( 8 )
                                    self.__Data[ 0 ].append( 0 )
                                    self.__Data[ 0 ].append( PayLoad[ 6 : 8 ]  )
                                    self.__Data[ 0 ].append( 1 )
                                    self.__Data[ 0 ].append( [ self.__AlarmNameList[ self.__AlarmType ][ 1 ] ] )
                                    # on regarde si produit gère des options l'alarme
                                    # ici le VMW Gaz
                                    if( self.__ProductType == SigfoxProductTypeCMV ) :
                                            # on regarde s'il s'agit de l'alarme sur l'entrée externe
                                            if self.__AlarmType == SigfoxalarmTypeExternalInput :
                                                    self.__Data.append( [] )
                                                    # unités des données
                                                    self.__Data[ 1 ].append( [ 'Pa' ] )
                                                    # taille de la donnée
                                                    self.__Data[ 1 ].append( 16 )
                                                    
                                                    self.__Data[ 1 ].append( 0 )
                                                    # données brutes
                                                    self.__Data[ 1 ].append( PayLoad[ 8 : 12 ]  )
                                                    # facteur de conversion
                                                    self.__Data[ 1 ].append( 1 )
                                                    # labels des données
                                                    self.__Data[ 1 ].append( [ 'Instant Pressure' ] )
                                    # ici le skydôme                                    
                                    elif( self.__ProductType == SigfoxProductTypeSkydome ) :
                                            # on regarde s'il s'agit de l'alarme sur l'entrée externe
                                            if self.__AlarmType == SigfoxalarmTypeOpeningState :
                                                    self.__Data.append( [] )
                                                    self.__Data[ 1 ].append( [ '°' ] )
                                                    self.__Data[ 1 ].append( 8 )
                                                    self.__Data[ 1 ].append( 0 )
                                                    self.__Data[ 1 ].append( PayLoad[ 8 : 10 ]  )
                                                    self.__Data[ 1 ].append( 1 )
                                                    self.__Data[ 1 ].append( [ 'Opening Angle' ] )
                                else :
                                    return False 
                            else :
                                return False
                        else :
                            raise "Data compression type is unknown"
                    else :
                        raise "Product type is unmanaged (" + str( self.__ProductType ) + ")"
                else :
                    return False
            # Ok création d'un objet sans Payload 
            return True
        #---------------------------------------------------------------------------------------------------------------
        # Add function to make an entire payload (1st frame and followings)
        #---------------------------------------------------------------------------------------------------------------
        def Add( self, PayLoad ) :
            # On vérifie la cohérence
            if(    ( PayLoad        != None )
               and ( len( PayLoad ) >= 4    ) ):
                # on récupère le type produit
                ProductType     = ( int( '0x' + PayLoad[ 0 ], 16 ) * 16 + int( '0x' + PayLoad[ 1 ], 16 ) )
                Byte            = ( int( '0x' + PayLoad[ 2 ], 16 ) * 16 + int( '0x' + PayLoad[ 3 ], 16 ) )
                MessageType     = ( Byte & 0x03 )
                CompressionType = ( Byte & 0x0C ) >> 2
                FrameIndex      = ( Byte & 0xF0 ) >> 4
                # on vérifie que l'on connait le produit
                if ProductType == self.__ProductType :
                    # on vérifie l'index
                    if FrameIndex == ( ( self.__FrameIndex + 1 ) % 16 ) :
                        # on note le nouvel index
                        self.__FrameIndex = FrameIndex
                        # On note le payload
                        self.__InputString  += PayLoad[ : ]
                        # on vérifie qu'il s'agit bien d'un message standard
                        if MessageType == self.__MessageType :
                            # on vérifie que le type de compression n'a pas changé
                            if CompressionType == self.__CompressionType :
                                # on regarde le nombre de listes
                                Number = len( self.__Data )
                                #print( 'DEBUG ' + str(Number) + ' liste(s)' )
                                # on cherche la structure à remplir
                                for Count in range( 0, Number ) :
                                        #print( 'DEBUG ' +  self.__Data[ Count ][ 3 ] )
                                        # on regarde ce que l'on peut copier par rapport à la taille
                                        if self.__Data[ Count ][ 2 ] != 0 :
                                                break
                                # on note la taille des données du payload
                                #print( 'DEBUG payload[4:] ' +  PayLoad[ 4: ] )
                                Size = int( len( PayLoad[ 4: ] ) / 2 )
                                #print( 'DEBUG size = ' + str( Size ) )
                                Pos  = 4
                                # on boucle tant qu'il y a de la donnée à copier
                                while Size > 0 :
                                        # on regarde ce que l'on peut copier (si taille > Size )
                                        if self.__Data[ Count ][ 2 ] >= Size :
                                                # on copie les données brutes
                                                #print( 'Copie les données brutes' )
                                                self.__Data[ Count ][ 3 ] += PayLoad[ Pos : ]
                                                # on décrémente la taille
                                                self.__Data[ Count ][ 2 ] -= Size
                                                #print( 'DEBUG self.__Data[ ' +str( Count ) + '][ 2 ] = ' + str( self.__Data[ Count ][ 2 ] ) )
                                                # ok
                                                return True
                                        # on copie juste le nombre de données nécessaires à la suite des données brutes
                                        self.__Data[ Count ][ 3 ] += PayLoad[ Pos : Pos + int( self.__Data[ Count ][ 2 ] ) * 2 ]
                                        # on décrémente aussi la taille
                                        Size                    -= self.__Data[ Count ][ 2 ]
                                        # on note là, où on en était
                                        Pos                     += self.__Data[ Count ][ 2 ] * 2
                                        # cette donnée est pleine
                                        self.__Data[ Count ][ 2 ]  = 0
                                        # on passe à la liste suivante
                                        Count                   += 1
                                        # on vérifie quand même
                                        if Count >= len( self.__Data ) :
                                                return False
                            else :
                                raise "Compression type is corrupted"
                        else :
                            return False
                    else :
                        return False
                else :
                    return False
        #---------------------------------------------------------------------------------------------------------------
        # This function decodes the entire payload [N.B.:The PayLoad must be complete] and returns the number of channel
        #---------------------------------------------------------------------------------------------------------------
        def Execute( self ) :
                # on boucle suivant le nombre de données à traiter
                for Count in range( 0, len( self.__Data ) ) :
                        # on regarde si l'on doit décompresser ou pas les données
                        if self.__Data[ Count ][ 1 ] == 0 :
                                # on crée une nouvelle dimension
                                self.__Array.append( [] )
                                # on regarde s'il faut inverser ou pas
                                self.__Array[ Count ].append( [ round( self.__Data[ Count ][ 2 ] * self.__Data[ Count ][ 4 ], 3 ) ] )
                                continue
                        # on regarde si l'on doit décompresser ou pas les données
                        elif self.__CompressionType == SigfoxMessageTypeCompressionNone :
                                # on stocke le payload
                                self.__SigfoxRawDecoder.Initialization( self.__Data[ Count ][ 1 ], self.__Data[ Count ][ 3 ] )
                                # on décode
                                self.__SigfoxRawDecoder.Execute()
                                # On stocke
                                #self.__Array.append( self.__SigfoxRawDecoder.Result()[ : ] )
                                # on crée une nouvelle dimension
                                self.__Array.append( [] )
                                # gestion de l'heure
                                Time = 0
                                # on regarde s'il faut inverser ou pas
                                for Cpt in range( 0, len( self.__SigfoxRawDecoder.Result() ) ) :
                                        # on regarde s'il y a un flux de données ou pas
                                        if self.__TimeUsed :
                                                # On calcule le delta
                                                Time = self.__MeasurePeriod * Cpt
                                                # on regarde si l'on doit inverse
                                                if self.__MeasureReverse :
                                                        Time = self.__MeasurePeriod * ( len( self.__SigfoxRawDecoder.Result() ) - 1 ) - Time
                                                # on insère aussi la donnée et l'heure
                                                self.__Array[ Count ].append( [ Time, round( self.__SigfoxRawDecoder.Result()[ Cpt ] * self.__Data[ Count ][ 4 ], 3 ) ] )
                                        else :
                                                self.__Array[ Count ].append( [ round( self.__SigfoxRawDecoder.Result()[ Cpt ] * self.__Data[ Count ][ 4 ], 3 ) ] )
                        # il faut décompresser les données
                        else :
                                # on stocke le payload
                                self.__SigfoxZipDecoder.Initialization( self.__Data[ Count ][ 1 ], self.__Data[ Count ][ 3 ] )
                                # on décode
                                self.__SigfoxZipDecoder.Execute()
                                # On stocke
                                #self.__Array.append( self.__SigfoxZipDecoder.Result()[ : ] )
                                # on crée une nouvelle dimension
                                self.__Array.append( [] )
                                # gestion de l'heure
                                Time = 0
                                # on regarde s'il faut inverser ou pas
                                for Cpt in range( 0, len( self.__SigfoxZipDecoder.Result() ) ) :
                                        # on regarde s'il y a un flux de données ou pas
                                        if self.__TimeUsed :
                                                # On calcule le delta
                                                Time = self.__MeasurePeriod * Cpt
                                                # on regarde si l'on doit inverse
                                                if self.__MeasureReverse :
                                                        Time = self.__MeasurePeriod * ( len( self.__SigfoxZipDecoder.Result() ) - 1 ) - Time
                                                # on insère aussi la donnée et l'heure
                                                self.__Array[ Count ].append( [ Time, round( self.__SigfoxZipDecoder.Result()[ Cpt ] * self.__Data[ Count ][ 4 ], 3 ) ] )
                                        else :
                                                self.__Array[ Count ].append( [ round( self.__SigfoxZipDecoder.Result()[ Cpt ] * self.__Data[ Count ][ 4 ], 3 ) ] )
                        # on regarde si l'on doit inverse
                        if self.__MeasureReverse :
                                self.__Array[ Count ].reverse()
                # on retourne le résultat
                return len( self.__Array )
        #---------------------------------------------------------------------------------------------------------------
        # This function gives the entire input string (string)
        #---------------------------------------------------------------------------------------------------------------
        def Input( self ) :
                # on retourne une erreur
                return self.__InputString
        #---------------------------------------------------------------------------------------------------------------
        # This function indicates whether there are more frame to load or not (boolean)
        #---------------------------------------------------------------------------------------------------------------
        def IsFullyLoaded( self ) :
                # On vérifie qu'il y a bien quelque chose dans l'objet
                if len( self.__Data ) == 0 : return False;
                # print( len( self.__Data ) )
                # pour intégrité
                for Count in range( 0, len( self.__Data ) ) :
                        # taille de la donnée
                        #print( 'DEBUG Taille ' + str(self.__Data[ Count ][ 1 ]) + ', Nombre données '+ str( self.__Data[ Count ][ 2 ]) ) 
						# nombre de données
                        print(  )
                        # on vériife que tout est bon
                        if(    ( self.__Data[ Count ][ 1 ] > 0 )
                           and ( self.__Data[ Count ][ 2 ] > 0 ) ):
                                # on retourne la valeur
                                return False
                # Return an error
                return True
        #---------------------------------------------------------------------------------------------------------------
        # This function gives the name of the product (string)
        #---------------------------------------------------------------------------------------------------------------
        def ProductName( self ) :
                # on va rechercher le bon nom
                for Name in self.__ProductNameList :
                        # on regarde si l'on trouve le nom
                        if( Name[ 0 ] == self.__ProductType ) :
                                # on retourne le nom
                                return Name[ 1 ]
                # on retourne le résultat
                return ''
        #---------------------------------------------------------------------------------------------------------------
        # This function gives the name of the message (string)
        #---------------------------------------------------------------------------------------------------------------
        def MessageName( self ) :
                # on va rechercher le bon nom
                for Name in self.__MessageNameList :
                        # on regarde si l'on trouve le nom
                        if( Name[ 0 ] == self.__MessageType ) :
                                # on retourne le nom
                                return Name[ 1 ]
                # on retourne le résultat
                return ''
        #---------------------------------------------------------------------------------------------------------------
        # This function gives the name of the alarm (string)
        #---------------------------------------------------------------------------------------------------------------
        def AlarmName( self ) :
                # on vérifie que l'on a bien une alarme
                if self.__MessageType == SigfoxMessageTypAlarm :
                        # on va rechercher le bon nom
                        for Name in self.__AlarmNameList :
                                # on regarde si l'on trouve le nom
                                if( Name[ 0 ] == self.__AlarmType ) :
                                        # on retourne le nom
                                        return Name[ 1 ]
                # on retourne le résultat
                return ''
        #---------------------------------------------------------------------------------------------------------------
        # This function gives the type of the product (integer) 
        #---------------------------------------------------------------------------------------------------------------
        def ProductType( self ) :
                # on retourne le résultat
                return self.__ProductType
        #---------------------------------------------------------------------------------------------------------------
        # This function gives the type of the message (integer) 
        #---------------------------------------------------------------------------------------------------------------
        def MessageType( self ) :
                # on retourne le résultat
                return self.__MessageType
        #---------------------------------------------------------------------------------------------------------------
        # This function gives the type of the alarm (integer) 
        #---------------------------------------------------------------------------------------------------------------
        def AlarmType( self ) :
                # on retourne le type
                return self.__AlarmType
        #---------------------------------------------------------------------------------------------------------------
        # This function gives the number of channels (integer)
        #---------------------------------------------------------------------------------------------------------------
        def ChannelNumber( self ) :
                # on retourne le résultat
                return len( self.__Data )
        #---------------------------------------------------------------------------------------------------------------
        # This function gives the aim the channels (function) (string)
        #---------------------------------------------------------------------------------------------------------------
        def ChannelFunctionOf( self, Index ) :
                # on regarde l'index
                if( Index < len( self.__Data ) ) :
                        # on retourne le résultat
                        return self.__Data[ Index ][ 5 ]
                else :
                        # on retourne une erreur
                        return None
        #---------------------------------------------------------------------------------------------------------------
        # This function gives the data unit the of the channel (string)
        #---------------------------------------------------------------------------------------------------------------
        def ChannelUnitOf( self, Index ) :
                # on regarde l'index
                if( Index < len( self.__Data ) ) :
                        # on retourne le résultat
                        return self.__Data[ Index ][ 0 ]
                else :
                        # on retourne une erreur
                        return None
        #---------------------------------------------------------------------------------------------------------------
        # This function gives all data of the channel ( array of float or integer )
        #---------------------------------------------------------------------------------------------------------------
        def ChannelDataOf( self, List ) :
                # pour intégrité
                if( List < len( self.__Array )  ) :
                        # on retourne le résultat
                        return self.__Array[ List ]
                # Erreur
                return None
        #---------------------------------------------------------------------------------------------------------------
        # This function gives a datum of a channel at the index( float or integer )
        #---------------------------------------------------------------------------------------------------------------
        def DataValue( self, List, Index ) :
                # pour intégrité
                if( List < len( self.__Array ) ) :
                        # on vériife que tout est bon
                        if( Index < len( self.__Array[ List ] ) ) :
                                # on retourne la valeur
                                return self.__Array[ List ][ Index ]
                # Return an error
                return None
        #---------------------------------------------------------------------------------------------------------------
        # This function returns the entire data of all channels at once
        #---------------------------------------------------------------------------------------------------------------
        def Result( self ) :
                # on retourne le résultat
                return self.__Array

          
